/*
 * Copyright (C) 2016 Moritz Orbach <zufall@apfelboymchen.net>
 * http://apfelboymchen.net/gnu/C/zufall/
 *
 * Zufall is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Zufall is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * calculates image backgrounjd color and size
 */
#include <stdio.h>
#include "bgcalc.h"

typedef struct {
	float bigger_ratio;
	float smaller_ratio;
	int *img_border;
	int *img_noborder;
	int *head_border;
	int *head_noborder;
	int *head_biggerside;
	int *crop_border;
	int *crop_noborder;
} borderpointer_t;



/*
 * returns the average of 2 colors as a string
 */
void nice_color(char *color, const color_t c1, const color_t c2)
{
	color[0] = '#';

	/* average into a string */
	snprintf(color+1, 3, "%.2x", (c1.red   + c2.red)   / 2);
	snprintf(color+3, 3, "%.2x", (c1.green + c2.green) / 2);
	snprintf(color+5, 3, "%.2x", (c1.blue  + c2.blue)  / 2);

	return;
}

static void screen_ratio(
		borderpointer_t *p,
		int * const head_w, int * const head_h, 
		int * const img_w,  int * const img_h,
		int * const crop_w, int * const crop_h
) {
	float isx_ratio, isy_ratio; /* image/screen */

	/* img size vs. screen size:
	 * >1: image is smaller (image fits more than one time onto the screen)
	 * <1: image is bigger  (image doesn't fit on the screen)
	 * The bigger ratio has the bigger border (imgsize fits more often into scrsize)
	 */
	isx_ratio = ((float)*head_w) / ((float)*img_w);
	isy_ratio = ((float)*head_h) / ((float)*img_h);

	/* set
	 * - smaller_ratio to smaller ratio (use ratio of small border)
	 * - border is set to be the image border of the smaller ratio
	 * - head_noborder is set to the head size of the smaller ratio
	 * - where the image has no border, the head has
	 */
	if (isy_ratio > isx_ratio) {
		/* bigger border is on top/bottom */
		p->smaller_ratio = isx_ratio;
		p->bigger_ratio = isy_ratio;
		p->img_noborder = img_h;
		p->crop_noborder = crop_h;
		p->head_noborder = head_h;
		p->img_border = img_w;
		p->crop_border = crop_w;
		p->head_border = head_w;
	} else {
		/* bigger border is on the sides */
		p->smaller_ratio = isy_ratio;
		p->bigger_ratio = isx_ratio;
		p->img_noborder = img_w;
		p->crop_noborder = crop_w;
		p->head_noborder = head_w;
		p->img_border = img_h;
		p->crop_border = crop_h;
		p->head_border = head_h;
	}
	if (*head_w > *head_h)
		p->head_biggerside = head_w;
	else
		p->head_biggerside = head_h;

}

/* ignore maxscale if the resulting img_border would be smaller minborder */
float option_minborder(int img_border, int head_noborder, float ratio, float maxscale, int minborder)
{
	if (ratio > maxscale && head_noborder - img_border * ratio < minborder)
		ratio = (float)head_noborder / (float)img_border;

	return ratio;
}
float option_maxscale(int img_border, float ratio, float maxscale)
{
	//printf("\tcorrect maxscale from %.2f ", ratio);
	/* don't make bigger than maxscale */
	if (img_border * ratio > img_border * maxscale)
		ratio = maxscale;
	//printf("to %.2f\n", ratio);
	return ratio;
}

void image_scaledsize(int head_w, int head_h, int *img_w, int *img_h, int *crop_w, int *crop_h, int maxscale_percent, int minborder, int crop)
{
	borderpointer_t p;
	p.head_border = NULL; /* size of head that will have no border */
	p.head_noborder = NULL; /* size of head that will have a border */
	p.img_border = NULL; /* size of image that will have a border */
	p.img_noborder = NULL; /* size of image that will have no border */

	float maxscale = (float)maxscale_percent / 100;


	//printf("\n"); // debug
	/*
	 * crop img
	 */
	screen_ratio(&p, &head_w, &head_h, img_w, img_h, crop_w, crop_h);
	float smaller_ratio = p.smaller_ratio;
	//float bigger_ratio = p.bigger_ratio;
	//printf("smaller sratio %.2f (%dx%d): %dx%d (%.2f)\n", smaller_ratio, head_w, head_h, *img_w, *img_h, (float)*img_w / (float)*img_h); //debug
	smaller_ratio = option_maxscale(*p.img_border, smaller_ratio, maxscale);
	float ir = (float)*p.img_noborder / (float)*p.img_border;
	/* don't crop into the same direction as the screen is oriented */
	printf("image|noborder: %d, border: %d (ratio %.2f)\n", *p.img_noborder, *p.img_border, ir); // debug
	//printf("head |noborder: %d, border: %d\n", *p.head_noborder, *p.head_border); // debug
	//printf("ratios: %.2f vs. %.2f\n", bigger_ratio, smaller_ratio);
	int bordersize =  (*p.head_noborder - *p.img_noborder * smaller_ratio) / 2;
	//printf("border: %dpx vs. %.2f\n", bordersize, *p.head_noborder * 0.35);

	int scaledown_acceptable = smaller_ratio >= 1 || *p.img_border / 4.9 < *p.img_noborder; /* small but equal scaledown is ok */
	int img_with_screen_orientation = p.head_border == p.head_biggerside;
	int tiny_border = bordersize < *p.head_noborder * 0.35; /* don't crop just to fill a tiny border */
	//printf("\e[033mwith a/r    : %d\nscaleok     : %d\nborder small: %d\n\e[0m", img_with_screen_orientation, scaledown_acceptable, tiny_border);
	/* scaling is ok if the conditions apply */
	if (crop == 0 || img_with_screen_orientation || scaledown_acceptable || tiny_border) {
		/*
		 * scale, no cropping
		 */
		//printf("from %dx%d ", *img_w, *img_h); // debug

		option_minborder(*p.img_border, *p.head_noborder, smaller_ratio, maxscale, minborder);
		*p.crop_noborder = *p.img_noborder;
		*p.crop_border = *p.img_border;

		/* scale */
		*img_w = *img_w * smaller_ratio;
		*img_h = *img_h * smaller_ratio;
		//printf("to %dx%d (%.2f, max %.2f), crop %dx%d\n", *img_w, *img_h, smaller_ratio, maxscale, *crop_w, *crop_h); // debug
	} else {
		/*
		 * crop and scale down
		 * the image would be scaled much too much - crop it so that it stays readable 
		 */

		/* scale to as big as possible on the noborder side */
		//float fullscale = (float)*p.head_noborder / (float)*p.img_noborder; /* ratio for scaling to full size */
		//printf("%.2f would scale the short side to full\n", fullscale);
		//fullscale = option_maxscale(*p.img_border, fullscale, maxscale);
		//
		// TODO landscape auch runterskalieren, oder wenn == screenheigt

		printf("\e[35mCROP\e[0m to %d from %d ", *p.head_noborder, *p.img_border); // debug1
		/* guess some good values based on image aspect ratio */
		float scale;
		float goodscale;
		if (ir < 0.15)
			goodscale = 1;
		else if (ir < 0.3)
			/* smaller */
			goodscale = 0.85;
		else
			/* smallest scaling */
			goodscale = 0.7;

		//scale = goodscale;
		if (goodscale > smaller_ratio)
			/* don't scale smaller than screen */
			scale = goodscale;
		else
			scale = smaller_ratio;
		printf("with possible ratios: %.2f vs. %.2f ", smaller_ratio, goodscale); // debug2

		/* corrections */
		//if (*p.head_noborder - *p.img_noborder < 100)
		//	/* very little of the image snipped off - annoying */
		//	*p.img_noborder = *p.head_noborder;

		printf("with ratio: %.2f (%d) ", scale, *p.img_noborder); // debug2

		/* crop (on unscaled image values) */
		*p.crop_noborder = *p.img_noborder;
		*p.crop_border = *p.head_border / scale;

		/* corrections */
		//if (*p.crop_border > *p.img_border)
		//	*p.crop_border = *p.img_border;
		//if (*p.crop_noborder > *p.img_noborder)
		//	*p.crop_noborder = *p.img_noborder;

		/* scale */
		*p.img_noborder = *p.img_noborder * scale;
		*p.img_border = *p.head_border;

		printf("to %dx%d\n", *p.img_border, *p.img_noborder); // debug3
	}

	printf("result: %dx%d crop %dx%d\n", *img_w, *img_h, *crop_w, *crop_h); // debug
	printf("\n"); // debug

	return;
}


