/*
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009
 * Moritz Orbach <zufall@apfelboymchen.net>
 *
 * Zufall is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Zufall is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * parses the command line. t_config cfg is extern for zufall.c.
 */

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>
#include <getopt.h>
#include <string.h>

#include "../config.h"

#include "list.h"
#include "cfg.h"
#include "memman.h"
#include "zerror.h"

t_config cfg;

static struct option long_options[] =
{
	{ "help",       no_argument,       NULL,  0  },
	{ "version",    no_argument,       NULL,  0  },
	{ "delay",      required_argument, NULL, 'd' },
	{ "maxscale",   required_argument, NULL, 's' },
	{ "minborder",  required_argument, NULL, 'b' },
	{ "crop",       no_argument,       NULL, 'C' },
	{ "minsize",    required_argument, NULL, 'm' },
	{ "command",    required_argument, NULL, 'c' },
	{ "no-set",     no_argument,       NULL, 'n' },
	{ "list",       required_argument, NULL, 'l' },
	{ "passes",     required_argument, NULL, 'p' },
	{ "sequential", no_argument,       NULL, 'S' },
	{ "multi-head", required_argument, NULL, 'M' },
	{ NULL, 0, NULL, 0 }
};


static void set_defaults()
{
	cfg.delay = 900; /* 15 minutes */
	cfg.maxscale = 150;
	cfg.minborder = 100;
	cfg.minsize = 0;
	cfg.setbg = 1;
	cfg.command = NULL;
	cfg.passes = -1;
	cfg.sequential = 0;
	cfg.mh_mode = PER_SCREEN;
	cfg.crop = 0;
}

/* ------ */

static void version(void)
{
	printf("%s\n", PACKAGE_STRING);
	printf("Written by %s\n", AUTHORS);
	printf("Send bugreports to <%s>\n", PACKAGE_BUGREPORT);
}
 
static void usage()
{
	printf("Usage: %s /path/to/imagedir-or-file [/path/to/imagedir-or-file]\n", PACKAGE_NAME);
	/* -pedantic: warning: string length `585' is greater than the length `509' ISO C89 compilers are required to support */
	printf(
			"  -d, --delay=DELAY          number of seconds an image is shown (default 900)\n"
			"  -M, multi-head=MODE        specifies the method of placing images if more than\n"
			"                             one monitor is available. MODE can be \"per_screen\",\n"
			"                             \"stretch\" or \"clone\" (default per_screen)\n"
	      );
	printf(
			"  -s  --maxscale=PERCENT     maximum PERCENT the image will be scaled\n"
			"                             (default 150)\n"
			"  -b  --minborder=PIXEL      ignore --maxscale if the remaining border would be\n"
			"                             smaller than --minborder (default 100)\n"
	      );
	printf(
			"  -m  --minsize=KILOBYTE     only show images bigger than KILOBYTE.\n"
			"                             (default 0)\n"
			"  -l  --list=FILE            load list of images from file\n"
                        "                             - format is one directory or file per line\n"
                        "                             - can also be a collection from gqview\n"
                        "                             - if FILE is \"-\", the list will be read from\n"
                        "                               stdin\n"
	      );

	printf(
			"  -S  --sequential           don't randomize the imagelist (useful with -l)\n"
			"  -c  --command=COMMAND      execute COMMAND on every picture change\n"
			"                             Within COMMAND\n"
			"                               %%f gets replaced with the image-filename\n"
			"                               %%c gets replaced with the background-color\n"
                        "                               %%w gets replaced with the scaled width\n"
                        "                               %%h gets replaced with the scaled height\n"
	      );
	printf(
                        "                               %%%% gets replaced with %%\n"
			"  -n  --no-set               don't set the background (but execute command\n"
			"                             if given)\n"
			"  -p  --passes=PASSES        exit after given number of images\n"
			"\n"
			"      --help                 display this help and exit\n"
			"      --version              output version information and exit\n"
			"\n"
			"Signals:\n"
			"  HUP                        change background\n"
			"  USR1                       info\n"
			"  TERM                       exit\n"
	      );
}


/*
 * string to int
 * returns
 *   0 if string is no number
 *   1 success
 */
static int s2int(char const *const str, signed long int *number_return)
{
	char *endptr;

	errno = 0;	/* To distinguish success/failure after call */
	*number_return = strtol(str, &endptr, 10);
	if (errno || *endptr != '\0') {
		if (errno) {
			/* strtol had a problem */
			printerrno("strtol");
		}
		return 0;
	}
	return 1;
}

/*
 * convert number into number_return, print error message, return sucess
 * Would suck for localization I guess, but that is not really needed right 
 * now.
 */
static int get_number(char const *const str, signed long int *number_return,
		      signed long int minimum, char *what)
{
	if (s2int(optarg, number_return)) {
		if (*number_return >= minimum)
			return 1;
		else {
			fprintf(stderr, "%s: %s too small. %ld is minimum.\n", PACKAGE, what, minimum);
			return 0;
		}
	} else {
		fprintf(stderr, "%s: invalid %s \"%s\"\n", PACKAGE, what, str);
		return 0;
	}
}

/*
 * interpret commandline
 * - malloc's memory for **collections
 *   must be freed by the caller.
 *
 * returns
 * - 0 on success
 * - 1 if the program should exit with success (--help etc.)
 * - 2 on error
 */
int cfg_cmdln(int const argc, char * const argv[], int *diroffset, char ***collections, int *collectionsc)
{
	char c;
	char **temp = NULL;	/* temporary array of pointers to the optarg strings */
	enum {
		success = 0,
		exit_success = 1,
		error = 2
	} cfg_rc;

	set_defaults();
	*collectionsc = 0;
	cfg_rc = success;
	int opt_index;
	while (cfg_rc == 0 && (c = getopt_long (argc, argv, "Sd:s:b:m:nc:l:p:M:C", long_options, &opt_index)) != -1)
		switch(c)
		{
			/* options without short options */
			case 0:
				/* information */
				if(strcmp( "help", long_options[opt_index].name ) == 0) {
					usage();
					cfg_rc = exit_success;
					break;
				}
				if(strcmp("version", long_options[opt_index].name) == 0) {
					version();
					cfg_rc = exit_success;
					break;
				}

			/* delay */
			case 'd': if (!get_number(optarg, &cfg.delay, 1, "delay"))
					  cfg_rc = error;
				  break;
			/* maxscale */
			case 's': if (!get_number(optarg, &cfg.maxscale, 1, "size"))
					  cfg_rc = error;
				  break;
			/* passes */
			case 'p': if (!get_number(optarg, &cfg.passes, 1, "number of passes"))
					  cfg_rc = error;
				  break;
			/* minsize */
			case 'm': if (!get_number(optarg, &cfg.minsize, 0, "minimum file size"))
						  cfg_rc = error;
					  else
						  cfg.minsize *= 1024; /* convert to byte */
				  break;
			/* minborder */
			case 'b': if (!get_number(optarg, &cfg.minborder, 0, "minimum border size"))
						  cfg_rc = error;
				  break;
			/* no-set */
			case 'n': cfg.setbg = 0;
				  break;
			/* crop */
			case 'C': cfg.crop = 1;
				  break;
			/* command */
			case 'c': cfg.command = optarg;
				  break;
			/* file */
			case 'l': temp = extendmem(temp, (*collectionsc + 1) * sizeof(*temp));
				  temp[*collectionsc] = optarg;
				  *collectionsc += 1;
				  break;
			/* sequential */
			case 'S': cfg.sequential = 1;
				  break;
			/* multi-head */
			case 'M': if (strcasecmp(optarg, "clone") == 0)
					  cfg.mh_mode = CLONE;
				  else if (strcasecmp(optarg, "stretch") == 0)
					  cfg.mh_mode = STRETCH;
				  else {
					  fprintf(stderr, "%s: invalid multi-head mode \"%s\"\n", PACKAGE, optarg);
					  cfg_rc = error;
				  }
				  break;
			case '?': /* errormessage comes from getopt_long */
				  cfg_rc = error;
		}

	if (cfg_rc == success && optind >= argc && *collectionsc == 0) {
		fprintf(stderr, "please specify a directory or image.\n");
		cfg_rc = error;
	}

	if (cfg_rc == success && cfg.setbg == 0 && cfg.command == NULL)
		/* the user's wish is my order */
		fprintf(stderr, "oh, well. Have fun ;)\n");

	*diroffset = optind;
	*collections = temp;

	return cfg_rc;
}
