/*
 * Copyright (C) 2004, 2005, 2007, 2008, 2009, 2016
 * Moritz Orbach <zufall@apfelboymchen.net>
 *
 * Zufall is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Zufall is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * interpret command string and execute commands
 *
 * exit(3) on serious programming errors
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "command.h"
#include "memman.h"
#include "zerror.h"

struct command_s {
	char *cmdtmpl;
	int invalid;
	char *errorstr;
}; /* command_t */


/*
 * returns position of c in array s,
 * or -1 if not found
 * or -1 if s is the last character
 * */
static unsigned int findformat(const char *s, char c)
{
	int i = 0;

	/* count up to end or c */
	while (s[i] != '\0' && s[i] != c)
		i++;

	/* return i if c is not last char (or \0) */
	if (s[i] != '\0' && s[i+1] != '\0')
		return i;
	else
		return -1;
}

/*
 * cats number to the end of the string
 * size of the string is expanded if needed
 * returns new size of string via size
 */
static void itoacat(char **dest, int src, size_t *size)
{
	int new;
	int ret;

	/* return size of new string, without \0 */
	new = snprintf(*dest, 0, "%d", src);
	if (new == 0)
		exit(3);

	/* ... without \0 ... */
	new++;

	*size += new;
	*dest = extendmem(*dest, *size);

	ret = snprintf(*dest + strlen(*dest), *size, "%d", src); 	/* return doesn't include \0 */

	if (ret != new - 1)
		/* this really shouldn't happen */
		exit(3);
}

/*
 * appends *src to **pdest, expanding size if necessary.
 * curlen is the current number of byte allocated for the string
 */
static void xstrcat (char **pdest, const char *src, size_t *curlen)
{
	int new;
	char *dest; /* doesn't work directly! FIXME ?? */

	if (src == NULL)
		/* NULL values are used to validate the template */
		return;

	dest = *pdest;

	/* return curlen of new string, without \0 */
	new = strlen(src);
	if (new < 1)
		return;

	/* append space for \0 */
	new++;

	*curlen += new;
	dest = extendmem(dest, *curlen);

	strncpy(dest + strlen(dest), src, new);
	/* strncpy: maximum "curlen" chars! */
	*pdest = dest;
}                                            

/*
 * appends count character from src to dest, extending dest if needed.
 *
 * curlen is the current number of byte allocated for the string
 * src must be null-terminated!
 * dest must be null-terminated!
 * dest will be null-terminated
 */
static void xstrncat(char **pdest, const char *src, size_t count, size_t *curlen)
{
	int new = 0;
	//char *dest;

	if (count < 1)
		exit(3);
	/* don't copy more than there is */
	if (count + 1 > strlen(src))
		exit(3);

	size_t len_dest = strlen(*pdest);
	if (len_dest + count + 1 > *curlen)  {
		new = (len_dest + count + 1) - *curlen;
		/* string has to be expanded */
		*curlen += new;
		*pdest = extendmem(*pdest, *curlen);
	}

	/* strlen needs a null-terminated string */
	(*pdest)[*curlen - 1] = '\0';
	strncpy(*pdest + strlen(*pdest), src, count);

	/* hammer bin ich doof! */
}                                            

static char *make_cmdline(command_t *command, char const *color, const char *image, int w, int h)
{
	char *cmdtmpl = command->cmdtmpl;
	char *cmdline;
	int i;
	size_t len = 1;
	int error = 0;

	cmdline = getmem(len);
	cmdline[0] = '\0';

	while ( !error && (i = findformat(cmdtmpl, '%')) >= 0 ) {

		xstrncat(&cmdline, cmdtmpl, i, &len); /* copy from start to %-1 */

		/* point to character after '%' */
		cmdtmpl = &cmdtmpl[++i]; /* safe, because findformat returns false if % is the last char */

		switch (*cmdtmpl) {
			case 'c': xstrcat(&cmdline, color, &len);
				  break;
			case 'i': /* compatibility */
			case 'f': xstrcat(&cmdline, image, &len);
				  break;
			case 'w': itoacat(&cmdline, w, &len);
				  break;
			case 'h': itoacat(&cmdline, h, &len);
				  break;
			case '%': xstrcat(&cmdline, "%", &len);
				  break;
			default : errorstr(&command->errorstr, "%%%c: invalid format character", *cmdtmpl);
				  error = 1;
				  break;
		}
		/* safe, findformat checked it */
		cmdtmpl++;
	}
	
	if (!error)
		/* append the rest */
		xstrcat(&cmdline, cmdtmpl, &len);
	else {
		freemem(cmdline);
		cmdline = NULL;
	}

	return cmdline;
}

/*
 * interface
 */

int command_execute(command_t *command, char const *color, const char *image, int w, int h)
{
	int ret;

	if (command->invalid) {
		return 0;
	}
	char *cmdline = make_cmdline(command, color, image, w, h);
	//printf("execute: %s\n", cmdline); // debug

	/* execute. fork/exec is not possible because the parameters must be splitted */
	ret = system(cmdline);
	freemem(cmdline);

	/* shell + c return-code have the opposite meaning */
	if (ret == 0)
		return 1;
	else
		return 0;

}

int command_valid(command_t *command)
{
	return command->invalid != 1;
}
char *command_errorstr(command_t *command)
{
	return command->errorstr;
}

command_t *command_init(char *cmdtmpl)
{
	command_t *command;
	command = getmem(sizeof(command_t));

	command->cmdtmpl = cmdtmpl;
	command->errorstr = getmem(9);
	strcpy(command->errorstr, "no error");
	char *cmdlinetest = make_cmdline(command, NULL, NULL, 0, 0);
	if (cmdlinetest == NULL)
		command->invalid = 1;
	else {
		freemem(cmdlinetest);
		command->invalid = 0;
	}

	return command;
}

void command_destroy(command_t *command)
{
	if (command != NULL) {
		if (command->errorstr != NULL)
			freemem(command->errorstr);
		freemem(command);

	}
}
