/*
 * Copyright (C) 2016 Moritz Orbach <zufall@apfelboymchen.net>
 *
 * Zufall is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Zufall is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Adds files that match specified conditions to a linked list.
 */

#include <stdio.h>
#include <string.h>
#include <strings.h>

#include <sys/stat.h>

#include "filesearch.h"
#include "memman.h"
#include "zerror.h"

/*
 * returns true if the filetype has an allowed extension
 * XXX wchar?
 */
static int extension_allowed(const char *file, filesearch_t *filesearch)
{
	char **s;
	char *ext;

	s = filesearch->allowed_extensions;

	ext = strrchr(file, '.');
	if (ext == NULL)
		return 0; /* no extension-dot */
	ext++; /* skip dot */
	if (ext == NULL)
		return 0; /* no extension */

	while (*s != NULL) {
		//printf("%s vs. %s\n", ext, *s); // debug
		if (!strcasecmp(ext, *s))
			return 1; /* extension_allowed */

		s = s + 1;
	}
	/* nothing found */
	return 0;
}

/*
 * Adds path to *filesearch.
 * If *sb is NULL a stat is done here.
 *
 * returns
 * - 0 on error
 * - 1 if the file was ignored
 * - 2 if the file was added
 */
int filesearch_add(filesearch_t *filesearch, char const * const path, struct stat *sb)
{
	struct stat lsb; /* use local struct stat if caller didn't pass one */

	if (sb == NULL) {
		if (stat(path, &lsb)) {
			printerrno(path);
			return 0;
		}
	} else
		lsb = *sb;

	if (lsb.st_size > filesearch->minsize && extension_allowed(path, filesearch)) {
		//printf("fs %p: add file: %s to %p\n", filesearch, path, filesearch->list); // debug
		list_append(filesearch->list, path, strlen(path) + 1);
		return 2;
	} else
		return 1;
}

list_t *filesearch_getlist(filesearch_t *filesearch)
{
	return filesearch->list;
}

void filesearch_destroy(filesearch_t *filesearch, int destroy_list)
{
	if (destroy_list)
		list_destroy(filesearch->list);
	freemem(filesearch);
}

/*
 * returned filesearch_t should be freed with filesearch_destroy()
 */
filesearch_t *filesearch_init(list_t *list, char **allowed_extensions, long int const minsize)
{
	filesearch_t *filesearch;

	if (list == NULL)
		list = list_init();
	filesearch = getmem(sizeof(filesearch_t));
	filesearch->list = list;
	filesearch->minsize = minsize;
	filesearch->allowed_extensions = allowed_extensions;

	return filesearch;
}
