/*
 * Copyright (C) 2004, 2005, 2006, 2007, 2009, 2016
 * Moritz Orbach <zufall@apfelboymchen.net>
 *
 * Zufall is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Zufall is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Keeps track of a running zufall.
 * - singnals
 * - currently shown images 
 */
#include <stdio.h> /* NULL */
#include <string.h>
#include "status.h"
#include "memman.h"
#include "list.h"

typedef struct {
	int head;
	char hexcolor[8]; // #123456\0
	int offset_x;
	int offset_y;
	int width;
	int height;
	double ratio;
	int cropped;
	int crop_x;
	int crop_y;
	int crop_w;
	int crop_h;
	char *path;
} imginfo_t;

struct status_s {
	reason_t reason;
	imginfo_t *current_image;
	list_t *imgstatus;
	int interrupt_reason; /* sig_atomic_t? */
	int running;
}; /* status_t */


/*
 * running status
 */

void status_stop(status_t *status)
{
	status->running = 0;
}
int status_isrunning(status_t *status)
{
	return status->running;
}
void status_break_sleep(status_t *status, reason_t reason)
{
	status->interrupt_reason = reason;
}
reason_t status_sleep_broken(status_t *status)
{
	return status->interrupt_reason;
}

/*
 * image information
 */

void status_remember_image(status_t *status, int head, char *imgpath)
{
	imginfo_t imginfo;

	imginfo.head  = head;
	imginfo.ratio = 0;

	if (status->imgstatus == NULL)
		status->imgstatus = list_init();
	imginfo_t *existing_slot;

	if ((existing_slot = list_get(status->imgstatus, head)) == NULL)
		status->current_image = list_append(status->imgstatus, &imginfo, sizeof (imginfo_t));
	else
		status->current_image = existing_slot;

	status->current_image->path = imgpath; /* a copy is not neccessary */
}
char *status_recall_name(status_t *status, int head)
{
	imginfo_t *i = list_get(status->imgstatus, head);
	return i->path;
}

void status_remember_color(status_t *status, char *hexcolor)
{
	strncpy(status->current_image->hexcolor, hexcolor, 8);
}
char *status_recall_color(status_t *status)
{
	return status->current_image->hexcolor;
}

void status_remember_offset(status_t *status, int x, int y)
{
	status->current_image->offset_x  = x;
	status->current_image->offset_y = y;
}
void status_recall_offset(status_t *status, int *w, int *h)
{
	*w = status->current_image->offset_x;
	*h = status->current_image->offset_y;
}

void status_remember_cropping(status_t *status, int x, int y, int w, int h)
{
	status->current_image->crop_x  = x;
	status->current_image->crop_y  = y;
	status->current_image->crop_w  = w;
	status->current_image->crop_h  = h;
}
void status_recall_cropping(status_t *status, int *x, int *y, int *w, int *h)
{
	*x = status->current_image->crop_x;
	*y = status->current_image->crop_y;
	*w = status->current_image->crop_w;
	*h = status->current_image->crop_h;
}

void status_remember_size(status_t *status, int w, int h)
{
	status->current_image->width  = w;
	status->current_image->height = h;
}
void status_recall_size(status_t *status, int *w, int *h)
{
	*w = status->current_image->width;
	*h = status->current_image->height;
}

void status_forget_images(status_t *status)
{
	if (status == NULL)
		return;
	if (status->imgstatus != NULL) {
		/* nothing rememberd, e.g. if the only file was not readable */
		list_destroy(status->imgstatus);
		status->imgstatus = NULL;
	}
}

/*
 * initialization
 */

void status_destroy(status_t *status)
{
	if (status == NULL)
		return;
	status_forget_images(status);
	freemem(status);
}

status_t *status_init(void)
{
	status_t *status;
	status = getmem(sizeof(status_t));

	status->imgstatus = NULL;
	status->interrupt_reason = NONE;
	status->running = 1;

	return status;
}
